#! /usr/bin/env python

import sys, re
import pysam

def get_seq(reference, chr, start, end):

    seq = ""
    for item in pysam.faidx(reference, chr + ":" + str(start) + "-" + str(end)):
        # if item[0] == ">": continue
        seq = seq + item.rstrip('\n')
    seq = seq.replace('>', '')
    seq = seq.replace(chr + ":" + str(start) + "-" + str(end), '')

    if re.search(r'[^ACGTNacgtn]', seq) is not None:
        print >> sys.stderr, "The return value in get_seq function includes non-nucleotide characters:"
        print >> sys.stderr, seq
        sys.exit(1)


    return seq


def reverse_complement(seq):
    complement = {'A': 'T', 'C': 'G', 'G': 'C', 'T': 'A',
                  'W': 'W', 'S': 'S', 'M': 'K', 'K': 'M', 'R': 'Y', 'Y': 'R',
                  'B': 'V', 'V': 'B', 'D': 'H', 'H': 'D', 'N': 'N'}

    return("".join(complement.get(base, base) for base in reversed(seq)))





#!/usr/bin/env python
"""
Simple python wrapper for SSW library
Please put the path of libssw.so into LD_LIBRARY_PATH or pass it explicitly as a parameter
By Yongan Zhao (March 2016)
"""

import sys
import os.path as op
import ctypes as ct



lBlosum50 = [
	#  A   R   N   D   C   Q   E   G   H   I   L   K   M   F   P   S   T   W   Y   V   B   Z   X   *
     	5, -2, -1, -2, -1, -1, -1,  0, -2, -1, -2, -1, -1, -3, -1,  1,  0, -3, -2,  0, -2, -1, -1, -5,	# A
       -2,  7, -1, -2, -4,  1,  0, -3,  0, -4, -3,  3, -2, -3, -3, -1, -1, -3, -1, -3, -1,  0, -1, -5,	# R
       -1, -1,  7,  2, -2,  0,  0,  0,  1, -3, -4,  0, -2, -4, -2,  1,  0, -4, -2, -3,  5,  0, -1, -5,	# N
       -2, -2,  2,  8, -4,  0,  2, -1, -1, -4, -4, -1, -4, -5, -1,  0, -1, -5, -3, -4,  6,  1, -1, -5,	# D
       -1, -4, -2, -4, 13, -3, -3, -3, -3, -2, -2, -3, -2, -2, -4, -1, -1, -5, -3, -1, -3, -3, -1, -5,	# C
       -1,  1,  0,  0, -3,  7,  2, -2,  1, -3, -2,  2,  0, -4, -1,  0, -1, -1, -1, -3,  0,  4, -1, -5,	# Q
       -1,  0,  0,  2, -3,  2,  6, -3,  0, -4, -3,  1, -2, -3, -1, -1, -1, -3, -2, -3,  1,  5, -1, -5,	# E
     	0, -3,  0, -1, -3, -2, -3,  8, -2, -4, -4, -2, -3, -4, -2,  0, -2, -3, -3, -4, -1, -2, -1, -5,	# G
       -2,  0,  1, -1, -3,  1,  0, -2, 10, -4, -3,  0, -1, -1, -2, -1, -2, -3,  2, -4,  0,  0, -1, -5,	# H
       -1, -4, -3, -4, -2, -3, -4, -4, -4,  5,  2, -3,  2,  0, -3, -3, -1, -3, -1,  4, -4, -3, -1, -5,	# I
       -2, -3, -4, -4, -2, -2, -3, -4, -3,  2,  5, -3,  3,  1, -4, -3, -1, -2, -1,  1, -4, -3, -1, -5,	# L
       -1,  3,  0, -1, -3,  2,  1, -2,  0, -3, -3,  6, -2, -4, -1,  0, -1, -3, -2, -3,  0,  1, -1, -5,	# K
       -1, -2, -2, -4, -2,  0, -2, -3, -1,  2,  3, -2,  7,  0, -3, -2, -1, -1,  0,  1, -3, -1, -1, -5,	# M
       -3, -3, -4, -5, -2, -4, -3, -4, -1,  0,  1, -4,  0,  8, -4, -3, -2,  1,  4, -1, -4, -4, -1, -5,	# F
       -1, -3, -2, -1, -4, -1, -1, -2, -2, -3, -4, -1, -3, -4, 10, -1, -1, -4, -3, -3, -2, -1, -1, -5,	# P
     	1, -1,  1,  0, -1,  0, -1,  0, -1, -3, -3,  0, -2, -3, -1,  5,  2, -4, -2, -2,  0,  0, -1, -5,	# S
    	0, -1,  0, -1, -1, -1, -1, -2, -2, -1, -1, -1, -1, -2, -1,  2,  5, -3, -2,  0,  0, -1, -1, -5, 	# T
       -3, -3, -4, -5, -5, -1, -3, -3, -3, -3, -2, -3, -1,  1, -4, -4, -3, 15,  2, -3, -5, -2, -1, -5, 	# W
       -2, -1, -2, -3, -3, -1, -2, -3,  2, -1, -1, -2,  0,  4, -3, -2, -2,  2,  8, -1, -3, -2, -1, -5, 	# Y
     	0, -3, -3, -4, -1, -3, -3, -4, -4,  4,  1, -3,  1, -1, -3, -2,  0, -3, -1,  5, -3, -3, -1, -5, 	# V
       -2, -1,  5,  6, -3,  0,  1, -1,  0, -4, -4,  0, -3, -4, -2,  0,  0, -5, -3, -3,  6,  1, -1, -5, 	# B
       -1,  0,  0,  1, -3,  4,  5, -2,  0, -3, -3,  1, -1, -4, -1,  0, -1, -2, -2, -3,  1,  5, -1, -5, 	# Z
       -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -5, 	# X
       -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5, -5,  1 	# *
       ]



class CAlignRes(ct.Structure):
    """
    @typedef	structure of the alignment result
    @field	nScore	the best alignment score
    @field	nScore2	sub-optimal alignment score
    @field	nRefBeg	0-based best alignment beginning position on reference;	ref_begin1 = -1 when the best alignment beginning
                                            position is not available
    @field	nRefEnd	0-based best alignment ending position on reference
    @field	nQryBeg	0-based best alignment beginning position on read; read_begin1 = -1 when the best alignment beginning
                                            position is not available
    @field	nQryEnd	0-based best alignment ending position on read
    @field	nRefEnd2	0-based sub-optimal alignment ending position on read
    @field	sCigar	best alignment cigar; stored the same as that in BAM format, high 28 bits: length, low 4 bits: M/I/D (0/1/2);
                                    cigar = 0 when the best alignment path is not available
    @field	nCigarLen	length of the cigar string; cigarLen = 0 when the best alignment path is not available
    """
    _fields_ = [('nScore', ct.c_uint16), 
                ('nScore2', ct.c_uint16), 
                ('nRefBeg', ct.c_int32), 
                ('nRefEnd', ct.c_int32), 
                ('nQryBeg', ct.c_int32), 
                ('nQryEnd', ct.c_int32), 
                ('nRefEnd2', ct.c_int32), 
                ('sCigar', ct.POINTER(ct.c_uint32)), 
                ('nCigarLen', ct.c_int32)] 



class CProfile(ct.Structure):
    """
    @typedef	structure of the query profile
    @field	pByte	byte array for profile
    @field	pWord	word array for profile
    @field	pRead	number array for read
    @field	pMat	score matrix
    @field	nReadLen	read length
    @field	nN	edge length of score matrix
    @field	nBias	bias
    """
    _fields_ = [('pByte', ct.POINTER(ct.c_int32)),
                ('pWord', ct.POINTER(ct.c_int32)),
                ('pRead', ct.POINTER(ct.c_int8)),
                ('pMat', ct.POINTER(ct.c_int8)),
                ('nReadLen', ct.c_int32),
                ('nN', ct.c_int32),
                ('nBias', ct.c_uint8)]



class CSsw(object):
    """
    A class for libssw
    """
    def __init__(self, sLibPath):
        """
        init all para
        @para   sLibpath    argparse object
        """
# load libssw
        sLibName = 'libssw.so'

        # modified by Yuichi Shiraishi
        # if not sLibPath:
        if sLibPath != "":

# user doesn't give the path explicitly
            # print op.join(sLibPath, sLibName)
            if not op.exists(op.join(sLibPath, sLibName)):
                print >> sys.stderr, 'libssw.so does not exist in the input path'
                sys.exit()
            self.ssw = ct.cdll.LoadLibrary(op.join(sLibPath,sLibName))
        else:
# otherwise just search in PATH
            bFound = False
            for s in sys.path:
                if op.exists(op.join(s,sLibName)):
                    bFound = True
                    self.ssw = ct.cdll.LoadLibrary(op.join(s,sLibName))
            if bFound == False:
                print >> sys.stderr, 'libssw.so does not exist in PATH'
                sys.exit()

# init ssw_init
        """
	@function	Create the query profile using the query sequence.
	@param	read	pointer to the query sequence; the query sequence needs to be numbers
	@param	readLen	length of the query sequence
	@param	mat	pointer to the substitution matrix; mat needs to be corresponding to the read sequence
	@param	n	the square root of the number of elements in mat (mat has n*n elements)
	@param	score_size	estimated Smith-Waterman score; if your estimated best alignment score is surely < 255 please set 0; if
						your estimated best alignment score >= 255, please set 1; if you don't know, please set 2
	@return	pointer to the query profile structure
	@note	example for parameter read and mat:
			If the query sequence is: ACGTATC, the sequence that read points to can be: 1234142
			Then if the penalty for match is 2 and for mismatch is -2, the substitution matrix of parameter mat will be:
			//A  C  G  T
			  2 -2 -2 -2 //A
			 -2  2 -2 -2 //C
			 -2 -2  2 -2 //G
			 -2 -2 -2  2 //T
			mat is the pointer to the array {2, -2, -2, -2, -2, 2, -2, -2, -2, -2, 2, -2, -2, -2, -2, 2}
        """
        self.ssw_init = self.ssw.ssw_init
        self.ssw_init.argtypes = [ct.POINTER(ct.c_int8), ct.c_int32, ct.POINTER(ct.c_int8), ct.c_int32, ct.c_int8]
        self.ssw_init.restype = ct.POINTER(CProfile)
# init init_destroy
        """
	@function	Release the memory allocated by function ssw_init.
	@param	p	pointer to the query profile structure
        """
        self.init_destroy = self.ssw.init_destroy
        self.init_destroy.argtypes = [ct.POINTER(CProfile)]
        self.init_destroy.restype = None
# init ssw_align
        """
!	@function	Do Striped Smith-Waterman alignment.
	@param	prof	pointer to the query profile structure
	@param	ref	pointer to the target sequence; the target sequence needs to be numbers and corresponding to the mat parameter of
				function ssw_init
	@param	refLen	length of the target sequence
	@param	weight_gapO	the absolute value of gap open penalty
	@param	weight_gapE	the absolute value of gap extension penalty
	@param	flag	bitwise FLAG; (from high to low) bit 5: when setted as 1, function ssw_align will return the best alignment
					beginning position; bit 6: when setted as 1, if (ref_end1 - ref_begin1 < filterd && read_end1 - read_begin1
					< filterd), (whatever bit 5 is setted) the function will return the best alignment beginning position and
					cigar; bit 7: when setted as 1, if the best alignment score >= filters, (whatever bit 5 is setted) the function
  					will return the best alignment beginning position and cigar; bit 8: when setted as 1, (whatever bit 5, 6 or 7 is
 					setted) the function will always return the best alignment beginning position and cigar. When flag == 0, only
					the optimal and sub-optimal scores and the optimal alignment ending position will be returned.
	@param	filters	score filter: when bit 7 of flag is setted as 1 and bit 8 is setted as 0, filters will be used (Please check the
 					decription of the flag parameter for detailed usage.)
	@param	filterd	distance filter: when bit 6 of flag is setted as 1 and bit 8 is setted as 0, filterd will be used (Please check
					the decription of the flag parameter for detailed usage.)
	@param	maskLen	The distance between the optimal and suboptimal alignment ending position >= maskLen. We suggest to use
					readLen/2, if you don't have special concerns. Note: maskLen has to be >= 15, otherwise this function will NOT
					return the suboptimal alignment information. Detailed description of maskLen: After locating the optimal
					alignment ending position, the suboptimal alignment score can be heuristically found by checking the second
					largest score in the array that contains the maximal score of each column of the SW matrix. In order to avoid
					picking the scores that belong to the alignments sharing the partial best alignment, SSW C library masks the
					reference loci nearby (mask length = maskLen) the best alignment ending position and locates the second largest
					score from the unmasked elements.
	@return	pointer to the alignment result structure
	@note	Whatever the parameter flag is setted, this function will at least return the optimal and sub-optimal alignment score,
			and the optimal alignment ending positions on target and query sequences. If both bit 6 and 7 of the flag are setted
			while bit 8 is not, the function will return cigar only when both criteria are fulfilled. All returned positions are
			0-based coordinate.
        """
        self.ssw_align = self.ssw.ssw_align
        self.ssw_align.argtypes = [ct.c_void_p, ct.POINTER(ct.c_int8), ct.c_int32, ct.c_uint8, ct.c_uint8, ct.c_uint8, ct.c_uint16, ct.c_int32, ct.c_int32]
        self.ssw_align.restype = ct.POINTER(CAlignRes)
# init align_destroy
        """
	@function	Release the memory allocated by function ssw_align.
	@param	a	pointer to the alignment result structure
        """
        self.align_destroy = self.ssw.align_destroy
        self.align_destroy.argtypes = [ct.POINTER(CAlignRes)]
        self.align_destroy.restype = None



def read_matrix(sFile):
    """
    read a score matrix for either DNA or protein
    assume the format of the input score matrix is the same as that of http://www.ncbi.nlm.nih.gov/Class/FieldGuide/BLOSUM62.txt

    """
    with open(args.sMatrix, 'r') as f:
        for l in f:
            if not l.startswith('#'):
                break
        lEle = l.strip().split()
        dEle2Int = {}
        dInt2Ele = {}
        for i,ele in enumerate(lEle):
            dEle2Int[ele] = i
            dEle2Int[ele.lower()] = i
            dInt2Ele[i] = ele
        nEleNum = len(lEle)
        lScore = []
        for l in f:
            lScore.extend([int(x) for x in l.strip().split()[1:]])

        return lEle, dEle2Int, dInt2Ele, lScore


if __name__ == '__main__':
    pass



#! /usr/bin/env python

import re, gzip
import pysam

def filterImproper(input_bam, output_bam, mapq_thres, keep_improper_pair):

    """
    This function is used for filtering short read with improper pairs,
    and low mapping qualities.
    """

    bamfile_in = pysam.AlignmentFile(input_bam, 'rb')
    bamfile_out = pysam.AlignmentFile(output_bam, 'wb', template = bamfile_in)

    for read in bamfile_in.fetch():
    
        # get the flag information
        flags = format(int(read.flag), "#014b")[:1:-1]

        # skip improper read pair
        if flags[1] != "1" and not keep_improper_pair: continue

        # skip if either of the read pair is unmapped
        if flags[2] == "1" or flags[3] == "1": continue

        # skip supplementary alignment
        if flags[8] == "1" or flags[11] == "1": continue

        # skip duplicated reads
        if flags[10] == "1": continue

        # skip if below the minimum mapping quality
        if (read.mapq < mapq_thres): continue


        bamfile_out.write(read)



def summarize_edge(edge_bed, edge_broaden_bed, output_file, margin):

    edge2count = {}
    edge_broaden2count = {}

    with open(edge_bed, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[len(F) - 1] != "1": continue
            key = F[12] + '\t' + '\t'.join(F[14:(len(F) - 1)])
            if key not in edge2count: edge2count[key] = 0
            edge2count[key] = edge2count[key] + 1


    with open(edge_broaden_bed, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[len(F) - 1] != str(margin * 2): continue
            if (F[16] == "donor" and F[17] == '+') or (F[16] == "acceptor" and F[17] == '-'): 
                F[13] = str(int(F[13]) + margin - 1)
                F[14] = str(int(F[14]) - margin)
            else:
                F[13] = str(int(F[13]) + margin)
                F[14] = str(int(F[14]) - margin + 1)

            key = F[12] + '\t' + '\t'.join(F[14:(len(F) - 1)])
            if key not in edge_broaden2count: edge_broaden2count[key] = 0
            edge_broaden2count[key] = edge_broaden2count[key] + 1


    hout = open(output_file, 'w')
    for key in list(set(edge2count.keys() + edge_broaden2count.keys())):
        count_edge = edge2count[key] if key in edge2count else 0
        count_edge_broaden = edge_broaden2count[key] if key in edge_broaden2count else 0
        print >> hout, key + '\t' + str(count_edge) + '\t' + str(count_edge_broaden)






#! /usr/bin/env python

import sys, re, subprocess

def generate_mutation_target(input_file, output_file, output_header, donor_size, acceptor_size):

    donor_size_exon, donor_size_intron = [int(x) for x in donor_size.split(',')]
    acceptor_size_intron, acceptor_size_exon = [int(x) for x in acceptor_size.split(',')]
    
    header2ind = {}
    hout = open(output_file + ".tmp", 'w')
    with open(input_file, 'r') as hin:
        
        header = hin.readline().rstrip('\n').split('\t')
        for (i, cname) in enumerate(header):
            header2ind[cname] = i

        hout_header = open(output_header, 'w')
        print >> hout_header, '\t'.join(header)
        hout_header.close()


        for line in hin:
            F = line.rstrip('\n').split('\t')

            boundary_pos = int(F[header2ind["Boundary_Pos"]])
            motif_type = F[header2ind["Motif_Type"]]
            strand = F[header2ind["Strand"]]

            # direct position
            if motif_type == "donor":
                if strand == '+':
                    start, end = boundary_pos - donor_size_exon, boundary_pos + donor_size_intron
                else:
                    start, end = boundary_pos - donor_size_intron - 1, boundary_pos + donor_size_exon - 1
            else:
                if strand == '+':
                    start, end = boundary_pos - acceptor_size_intron - 1, boundary_pos + acceptor_size_exon - 1
                else:
                    start, end = boundary_pos - acceptor_size_exon, boundary_pos + acceptor_size_intron

            print >> hout, '\t'.join([F[header2ind["Chr"]], str(start), str(end), "direct-impact"] + F)

            # opposite-side-impact 
            unique_junc_list = list(set(F[header2ind["Junction_List"]].split(',')))
            for junc in sorted(unique_junc_list):

                junc_match = re.match(r'([^ \t\n\r\f\v,]+)\:(\d+)\-(\d+)', junc)
                if junc_match is None: 
                    print >> sys.stderr, "junction mismatch at " + '\n' + '\t'.join(F)
                    sys.exit(1)

                opposite_motif_type = "donor" if motif_type == "acceptor" else "acceptor"

                if (motif_type == "donor" and strand == '+') or (motif_type == "acceptor" and strand == '-'):
                    opposite_boundary_pos = int(junc_match.group(3))
                else:
                    opposite_boundary_pos = int(junc_match.group(2)) 

                if opposite_motif_type == "donor":
                    if strand == '+':
                        start, end = opposite_boundary_pos - donor_size_exon, opposite_boundary_pos + donor_size_intron
                    else:
                        start, end = opposite_boundary_pos - donor_size_intron - 1, opposite_boundary_pos + donor_size_exon - 1
                else:   
                    if strand == '+':
                        start, end = opposite_boundary_pos - acceptor_size_intron - 1, opposite_boundary_pos + acceptor_size_exon - 1
                    else:
                        start, end = opposite_boundary_pos - acceptor_size_exon, opposite_boundary_pos + acceptor_size_intron
                        
                print >> hout, '\t'.join([F[header2ind["Chr"]], str(start), str(end), "opposite-side-impact"] + F)
 
    hout.close()

    hout = open(output_file, 'w')
    subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k3,3n", output_file + ".tmp"], stdout = hout)
    hout.close()

    subprocess.check_call(["rm", "-rf", output_file + ".tmp"])


def generate_sv_target(input_file, output_file, output_header, intron_margin):

    header2ind = {}
    hout = open(output_file + ".tmp", 'w')
    with open(input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for (i, cname) in enumerate(header):
            header2ind[cname] = i

        hout_header = open(output_header, 'w')
        print >> hout_header, '\t'.join(header)
        hout_header.close()

        for line in hin:
            F = line.rstrip('\n').split('\t')

            unique_junc_list = list(set(F[header2ind["Junction_List"]].split(',')))
            for junc in sorted(unique_junc_list):

                junc_match = re.match(r'([^ \t\n\r\f\v,]+)\:(\d+)\-(\d+)', junc)
                if junc_match is None:
                    print >> sys.stderr, "junction mismatch at " + '\n' + '\t'.join(F)
                    sys.exit(1)

                jchr, jstart, jend = junc_match.group(1), int(junc_match.group(2)), int(junc_match.group(3))

                print >> hout, '\t'.join([jchr, str(jstart - intron_margin), str(jend + intron_margin)] + F)

    hout.close()


    hout = open(output_file, 'w')
    subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k3,3n", output_file + ".tmp"], stdout = hout)
    hout.close()

    subprocess.check_call(["rm", "-rf", output_file + ".tmp"])


def process_result(input_file, intron_retention_header_file, output_file, donor_size, acceptor_size):

    donor_size_exon, donor_size_intron = [int(x) for x in donor_size.split(',')]
    acceptor_size_intron, acceptor_size_exon = [int(x) for x in acceptor_size.split(',')]


    hout = open(output_file, 'w')

    with open(intron_retention_header_file, 'r') as hin:
        intron_retention_header = hin.readline().rstrip('\n')
        

    print >> hout, intron_retention_header + '\t' + '\t'.join(["Mutation_Key", "Motif_Pos", "Mutation_Type", "Is_Canonical", "Intron_Retention_Type"])

    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            mutation_key = F[3]
            motif_pos = F[4] + ':' + str(int(F[5]) + 1) + '-' + F[6] + ',' + F[12]
            
            if (F[11] == "donor" and F[7] == "direct-impact") or (F[11] == "acceptor" and F[7] == "opposite-side-impact"):
                motif_type = "splicing donor disruption"
            elif (F[11] == "acceptor" and F[7] == "direct-impact") or (F[11] == "donor" and F[7] == "opposite-side-impact"): 
                motif_type = "splicing acceptor disruption"

            # check cannonical
            mut_start, mut_end = int(F[1]) + 1, int(F[2])
            if motif_type == "splicing donor disruption":
                if F[12] == '+':
                    canonical_start, canonical_end = int(F[5]) + donor_size_exon + 1, int(F[5]) + donor_size_exon + 2
                else:
                    canonical_start, canonical_end = int(F[5]) + donor_size_intron - 1, int(F[5]) + donor_size_intron 
            else:
                if F[12] == '+':
                    canonical_start, canonical_end = int(F[5]) + acceptor_size_intron - 1, int(F[5]) + acceptor_size_intron
                else:
                    canonical_start, canonical_end = int(F[5]) + acceptor_size_exon + 1, int(F[5]) + acceptor_size_exon + 2

            is_canonical = "canonical" if (mut_start <= canonical_end and mut_end >= canonical_start) else "non-canonical" 

            print >> hout, '\t'.join(F[8:]) + '\t' + mutation_key + '\t' + motif_pos + '\t' + motif_type + '\t' + is_canonical + '\t' + F[7]

    hout.close()



def process_result_sv(input_file, intron_retention_header_file, output_file):

    hout = open(output_file, 'w')

    with open(intron_retention_header_file, 'r') as hin:
        intron_retention_header = hin.readline().rstrip('\n')


    print >> hout, intron_retention_header + '\t' + "SV_Key"

    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            sv_key = F[4]

            """
            # check direction consistency
            consistent_flag = 0

            if F[11] not in ["donor", "acceptor"]: print "error!!"
            if F[12] not in ['+', '-']: print "error!!"

            if F[11] == "donor" and F[12] == "+" and F[3] == "-": consistent_flag = 1
            if F[11] == "donor" and F[12] == "-" and F[3] == "+": consistent_flag = 1
            if F[11] == "acceptor" and F[12] == "+" and F[3] == "+": consistent_flag = 1
            if F[11] == "acceptor" and F[12] == "-" and F[3] == "-": consistent_flag = 1

            if consistent_flag == 1: continue
            """
            print >> hout, '\t'.join(F[8:]) + '\t' + sv_key

    hout.close()




#! /usr/bin/env python

import sys, re, pysam
import my_seq

mut_seq_margin = 100

# not UCSC bed format (-1 based start positions)
def generate_template_seq(output_file, reference, mut_chr, mut_start, mut_end, mut_ref, mut_alt, 
                          motif_chr, motif_start, motif_end, motif_type, motif_strand,
                          junc_list, donor_size, acceptor_size, template_size):

    donor_size_exon, donor_size_intron = [int(x) for x in donor_size.split(',')]
    acceptor_size_intron, acceptor_size_exon = [int(x) for x in acceptor_size.split(',')]

    unique_junc_list = list(set(junc_list.split(',')))

    if motif_type == "donor":
        if motif_strand == '+':
            motif_exon_start, motif_exon_end = motif_start, motif_start + donor_size_exon - 1
            motif_intron_start, motif_intron_end = motif_start + donor_size_exon, motif_start + donor_size_exon + donor_size_intron - 1
        else:
            motif_intron_start, motif_intron_end = motif_start, motif_start + donor_size_intron - 1
            motif_exon_start, motif_exon_end = motif_start + donor_size_intron, motif_start + donor_size_exon + donor_size_intron - 1
    else: # acceptor
        if motif_strand == '+':
            motif_intron_start, motif_intron_end = motif_start, motif_start + acceptor_size_intron - 1
            motif_exon_start, motif_exon_end = motif_start + acceptor_size_intron, motif_start + acceptor_size_exon + acceptor_size_intron - 1
        else:
            motif_exon_start, motif_exon_end = motif_start, motif_start + acceptor_size_exon - 1
            motif_intron_start, motif_intron_end = motif_start + acceptor_size_exon, motif_start + acceptor_size_exon + acceptor_size_intron - 1

    key2seq = {}  
    # annotated splice junction without mutation
    cnum = 0
    for junc in sorted(unique_junc_list):
        junc_match = re.match(r'([^ \t\n\r\f\v,]+)\:(\d+)\-(\d+)', junc)
        junc_chr, junc_start, junc_end = junc_match.group(1), int(junc_match.group(2)), int(junc_match.group(3))

        seq = my_seq.get_seq(reference, junc_chr, junc_start - template_size + 1, junc_start) + \
              my_seq.get_seq(reference, junc_chr, junc_end, junc_end + template_size - 1)
        key2seq["splice_junction_negative_" + str(cnum)] = seq
        cnum = cnum + 1


    # annotated splice junction with mutation (only when mutations occur within exonic motif region)
    if mut_start >= motif_exon_start and mut_end <= motif_exon_end:
        cnum = 0
        for junc in sorted(unique_junc_list):
            junc_match = re.match(r'([^ \t\n\r\f\v,]+)\:(\d+)\-(\d+)', junc)
            junc_chr, junc_start, junc_end = junc_match.group(1), int(junc_match.group(2)), int(junc_match.group(3))

            if (motif_type == "donor" and motif_strand == '+') or (motif_type == "acceptor" and motif_strand == '-'):
                mut_seq_tmp = my_seq.get_seq(reference, junc_chr, junc_start - mut_seq_margin + 1, junc_start)
                mut_start_rel, mut_end_rel = mut_start - junc_start + mut_seq_margin - 1, mut_end - junc_start + mut_seq_margin - 1
            else:
                mut_seq_tmp = my_seq.get_seq(reference, junc_chr, junc_end, junc_end + mut_seq_margin - 1)
                mut_start_rel, mut_end_rel = mut_start - junc_end, mut_end - junc_end

            # for debug
            if mut_ref != '-' and mut_seq_tmp[mut_start_rel:(mut_end_rel + 1)] != mut_ref:
                print >> sys.stderr, '\t'.join([mut_chr, str(mut_start), str(mut_end), mut_ref, mut_alt, junc])
                print >> sys.stderr, '\t'.join([mut_seq_tmp[mut_start_rel:(mut_end_rel + 1)], mut_ref])
                print >> sys.stderr, "mutation inconsistent!!!"
                sys.exit(1) 

            # SNV
            if mut_ref != '-' and mut_alt != '-': mut_seq_tmp = mut_seq_tmp[:mut_start_rel] + mut_alt + mut_seq_tmp[(mut_end_rel + 1):]

            # deletion
            if mut_alt == '-': mut_seq_tmp = mut_seq_tmp[:mut_start_rel] + mut_seq_tmp[(mut_end_rel + 1):]

            # insertion
            if mut_ref == '-': mut_seq_tmp = mut_seq_tmp[:(mut_start_rel + 1)] + mut_alt + mut_seq_tmp[(mut_start_rel + 1):]

            if (motif_type == "donor" and motif_strand == '+') or (motif_type == "acceptor" and motif_strand == '-'):
                seq = mut_seq_tmp[(-template_size):] + my_seq.get_seq(reference, junc_chr, junc_end, junc_end + template_size - 1)
            else:
                seq = my_seq.get_seq(reference, junc_chr, junc_start - template_size + 1, junc_start) + mut_seq_tmp[:(template_size)]

            key2seq["splice_junction_positive_" + str(cnum)] = seq
            cnum = cnum + 1

            
    # intron retention without mutation
    if (motif_type == "donor" and motif_strand == '+') or (motif_type == "acceptor" and motif_strand == '-'):
        seq_left_tmp = my_seq.get_seq(reference, junc_chr, motif_exon_end - mut_seq_margin + 1, motif_exon_end)
        seq_right_tmp = my_seq.get_seq(reference, junc_chr, motif_intron_start, motif_intron_start + mut_seq_margin - 1)
        boundary_pos = motif_exon_end
    else:
        seq_left_tmp = my_seq.get_seq(reference, junc_chr, motif_intron_end - mut_seq_margin + 1, motif_intron_end)
        seq_right_tmp = my_seq.get_seq(reference, junc_chr, motif_exon_start, motif_exon_start + mut_seq_margin - 1)
        boundary_pos = motif_intron_end
    key2seq["intron_retention_negative"] = seq_left_tmp[(-template_size):] + seq_right_tmp[:(template_size)]

   
    # intron retention with mutation
    mut_seq_left_tmp, mut_seq_right_tmp = seq_left_tmp, seq_right_tmp

    # in this case, we remove nucleotides from concatenated sequences
    mut_start_rel, mut_end_rel = mut_start - boundary_pos + mut_seq_margin - 1, mut_end - boundary_pos + mut_seq_margin - 1
    mut_seq_tmp = mut_seq_left_tmp + mut_seq_right_tmp


    # SNV
    if mut_ref != '-' and mut_alt != '-': 

        # for debug
        if mut_seq_tmp[mut_start_rel] != mut_ref:
            print '\t'.join([mut_chr, str(mut_start), str(mut_end), mut_ref, mut_alt])
            print mut_seq_tmp
            print mut_start_rel
            print >> sys.stderr, "mutation inconsistent!!!"
            sys.exit(1)
        mut_seq_tmp = mut_seq_tmp[:mut_start_rel] + mut_alt + mut_seq_tmp[(mut_end_rel + 1):]

        mut_seq_start_pos = mut_seq_margin - template_size
        mut_seq_end_pos = mut_seq_start_pos + 2 * template_size
        key2seq["intron_retention_positive"] = mut_seq_tmp[mut_seq_start_pos:mut_seq_end_pos]


    elif mut_alt == '-': # deletion

        # for debug
        if mut_seq_tmp[mut_start_rel:(mut_end_rel + 1)] != mut_ref != '-':
            print '\t'.join([mut_chr, str(mut_start), str(mut_end), mut_ref, mut_alt])
            print >> sys.stderr, "mutation inconsistent!!!"
            sys.exit(1)
        mut_seq_tmp = mut_seq_tmp[:mut_start_rel] + mut_seq_tmp[(mut_end_rel + 1):]
 
        del_size_left = max(0, (min(boundary_pos, mut_end) - mut_start))
        mut_seq_start_pos = mut_seq_margin - template_size - del_size_left
        mut_seq_end_pos = mut_seq_start_pos + 2 * template_size

        key2seq["intron_retention_positive"] = mut_seq_tmp[mut_seq_start_pos:mut_seq_end_pos]


    elif mut_ref == '-': #insertion
     
        mut_seq_tmp = mut_seq_tmp[:(mut_start_rel + 1)] + mut_alt + mut_seq_tmp[(mut_start_rel + 1):]
        ins_size_left = len(mut_alt)
        mut_seq_start_pos = mut_seq_margin - template_size + ins_size_left
        mut_seq_end_pos = mut_seq_start_pos + 2 * template_size
        key2seq["intron_retention_positive"] = mut_seq_tmp[mut_seq_start_pos:mut_seq_end_pos]


    hout = open(output_file, 'w')
    for key in sorted(key2seq):
        print >> hout, ">" + key + '\n' + key2seq[key]
    
    hout.close()
 

    
def extract_read_around_boundary(bam_file, output_file, reference, motif_chr, motif_start, motif_end, read_search_margin):

    bamfile = pysam.Samfile(bam_file, 'rb')
    hout = open(output_file, 'w') 
    for read in bamfile.fetch(motif_chr, max(0, motif_start - read_search_margin), motif_end + read_search_margin):

        # get the flag information
        flags = format(int(read.flag), "#014b")[:1:-1]

        # skip unmapped read 
        if flags[2] == "1" or flags[3] == "1": continue 

        # skip supplementary alignment
        if flags[8] == "1" or flags[11] == "1": continue

        # skip duplicated reads
        if flags[10] == "1": continue

        read_id = read.qname + '/1' if flags[6] == '1' else read.qname + '/2'
        
        print >> hout, '>' + read_id + '\n' + read.seq

    bamfile.close()
    hout.close()
 



#! /usr/bin/env python

import sys, subprocess
import pysam
import my_seq

def anno2vcf(input_file, output_file, reference):

    hin = open(input_file, 'r')
    hout = open(output_file, 'w')

    for line in hin:
        F = line.rstrip('\n').split('\t')
        if F[0].startswith('#'): continue
        if F[0] == "Chr" and F[1] == "Start" and F[2] == "End" and F[3] == "Ref" and F[4] == "Alt": continue

        pos, ref, alt = F[1], F[3], F[4]
        
        # insertion
        if F[3] == "-":
            # get the sequence for the reference base
            seq = my_seq.get_seq(reference, F[0], int(F[1]), int(F[1]))
            ref, alt = seq, seq + F[4]

        # deletion
        if F[4] == "-":
            # get the sequence for the reference base
            seq = my_seq.get_seq(reference, F[0], int(F[1]) - 1, int(F[1]) - 1)
            pos, ref, alt = str(int(F[1]) - 1), seq + F[3], seq

        # QUAL = int(float(F[15]) * 10)
        QUAL = 60
        # INFO = "TD=" + F[9] + ";TV=" + F[10] + ";ND=" + F[13] + ";NV=" + F[14] + ";SOMATIC"
        INFO = "SOMATIC"

        print >> hout, F[0] + "\t" + pos + "\t.\t" + ref + "\t" + alt \
            + "\t" + str(QUAL) + "\t" + "PASS" + "\t" + INFO 


def remove_vcf_header(input_file, output_file):

    hout = open(output_file, 'w')
    with open(input_file, 'r') as hin:
        for line in hin:
            line = line.rstrip('\n')
            if not line.startswith('#'): print >> hout, line

    hout.close()


def vcf2bed(mutation_file, output_file):

    hout = open(output_file, 'w')
    with open(mutation_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            tchr, tstart, tend = F[0], str(int(F[1]) - 1), F[1]

            # deletion
            if len(F[3]) > 1:
                tstart, tend = F[1], str(int(F[1]) + len(F[3]) - 1)
            # insertion
            elif len(F[4]) > 1:
                tstart, tend = str(int(F[1]) - 1), F[1]
            # substitution
            elif len(F[3]) == 1 and len(F[4]) == 1:
                tstart, tend = str(int(F[1]) - 1), F[1]
 
            print >> hout, tchr + '\t' + tstart + '\t' + tend + '\t' + ','.join([F[0], F[1], F[3], F[4]])

    hout.close()


def anno2bed(mutation_file, output_file):

    hout = open(output_file + ".tmp", 'w')
    with open(mutation_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            if F[0].startswith('#'): continue
            if F[0] == "Chr" and F[1] == "Start" and F[2] == "End" and F[3] == "Ref" and F[4] == "Alt": continue

            print >> hout, '\t'.join([F[0], str(int(F[1]) - 1), F[2], F[3], F[4]])

    hout.close()

    hout = open(output_file, 'w')
    subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k3,3n", output_file + ".tmp"], stdout = hout)
    hout.close()

    subprocess.check_call(["rm", "-rf", output_file + ".tmp"])


def genosv2bed(input_file, output_file):

    hout = open(output_file, 'w')
    num = 1
    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if F[0].startswith('#'): continue
            if F[0] == "Chr_1" and F[1] == "Pos_1": continue
            chr1, chr2 = F[0], F[3]
            start1, end1 = str(int(F[1]) - 1), F[1]
            start2, end2 = str(int(F[4]) - 1), F[4]
            dir1, dir2 = F[2], F[5]
            inseq = F[6]

            key = ','.join([chr1, end1, dir1, chr2, end2, dir2, inseq])

            print >> hout, '\t'.join([chr1, start1, end1, dir1, key])
            print >> hout, '\t'.join([chr2, start2, end2, dir2, key])

    hout.close()





#! /usr/bin/env python

import pysam

def filter_intron_retention(input_file, output_file, control_file, num_thres, ratio_thres):

    is_control = True if control_file is not None else False
    if is_control: control_db = pysam.TabixFile(control_file)

    header2ind = {}
    hout = open(output_file, 'w')
    with open(input_file, 'r') as hin:

        header = hin.readline().rstrip('\n').split('\t')
        for i, cname in enumerate(header):
            header2ind[cname] = i

        print >> hout, '\t'.join(header)

        for line in hin:

            F = line.rstrip('\n').split('\t')
            if int(F[header2ind["Intron_Retention_Read_Count"]]) < num_thres: continue

            intron_ratio = 0
            if F[header2ind["Edge_Read_Count"]] != "0":
                intron_ratio = float(F[header2ind["Intron_Retention_Read_Count"]]) / float(F[header2ind["Edge_Read_Count"]])

            if intron_ratio < ratio_thres: continue


            ##########
            # remove control files
            if is_control:
                tabixErrorFlag = 0
                try:
                    records = control_db.fetch(F[0], int(F[1]) - 5, int(F[1]) + 5)
                except Exception as inst:
                    # print >> sys.stderr, "%s: %s" % (type(inst), inst.args)
                    # tabixErrorMsg = str(inst.args)
                    tabixErrorFlag = 1

                control_flag = 0;
                if tabixErrorFlag == 0:
                    for record_line in records:
                        record = record_line.split('\t')
                        if F[0] == record[0] and F[1] == record[1] and F[2] == record[2]:
                            control_flag = 1

                if control_flag == 1: continue

            print >> hout, '\t'.join(F)
 

    hout.close()





#!/usr/bin/env python
"""
Simple python wrapper for SSW library
Please put the path of libssw.so into LD_LIBRARY_PATH or pass it explicitly as a parameter
By Yongan Zhao (March 2016)
"""

import sys, os, re
import os.path as op
import argparse as ap
import ctypes as ct
import timeit as ti
import gzip
import math

import ssw_lib




def read(sFile):
    """
    read a sequence file
    @param  sFile   sequence file
    """
    def read_one_fasta(f):
        """
        read a fasta file
        @param  f   file handler
        """
        sId = ''
        sSeq = ''
        for l in f:
            if l.startswith('>'):
                if sSeq:
                    yield sId, sSeq, ''
                sId = l.strip()[1:].split()[0]
                sSeq = ''
            else:
                sSeq += l.strip()

        yield sId, sSeq, ''

    def read_one_fastq(f):
        """
        read a fastq file
        @param  f   file handler
        """
        sId = ''
        sSeq = ''
        s3 = ''
        sQual = ''
        for l in f:
            sId = l.strip()[1:].split()[0]
            sSeq = f.next()
            s3 = f.next()
            sQual = f.next()

            yield sId, sSeq, sQual

# test if fasta or fastq
    bFasta = True
    ext = op.splitext(sFile)[1][1:].strip().lower()
    if ext == 'gz' or ext == 'gzip':
        with gzip.open(sFile, 'r') as f:
            l = f.next()
            if l.startswith('>'):
                bFasta = True
            elif l.startswith('@'):
                bFasta = False
            else:
                print >> sys.stderr, 'file format cannot be recognized'
                sys.exit()
    else:
        with open(sFile, 'r') as f:
            l = f.next()
            if l.startswith('>'):
                bFasta = True
            elif l.startswith('@'):
                bFasta = False
            else:
                print >> sys.stderr, 'file format cannot be recognized'
                sys.exit()

# read
    if ext == 'gz' or ext == 'gzip':
        with gzip.open(sFile, 'r') as f:
            if bFasta == True:
                for sId,sSeq,sQual in read_one_fasta(f):
                    yield sId, sSeq, sQual
            else:
                for sId,sSeq,sQual in read_one_fastq(f):
                    yield sId, sSeq, sQual
    else:
        with open(sFile, 'r') as f:
            if bFasta == True:
                for sId,sSeq,sQual in read_one_fasta(f):
                    yield sId, sSeq, sQual
            else:
                for sId,sSeq,sQual in read_one_fastq(f):
                    yield sId, sSeq, sQual


def to_int(seq, lEle, dEle2Int):
    """
    translate a sequence into numbers
    @param  seq   a sequence
    """
    num_decl = len(seq) * ct.c_int8
    num = num_decl()
    for i,ele in enumerate(seq):
        try:
            n = dEle2Int[ele]
        except KeyError:
            n = dEle2Int[lEle[-1]]
        finally:
            num[i] = n

    return num


def align_one(ssw, qProfile, rNum, nRLen, nOpen, nExt, nFlag, nMaskLen):
    """
    align one pair of sequences
    @param  qProfile   query profile
    @param  rNum   number array for reference
    @param  nRLen   length of reference sequence
    @param  nFlag   alignment flag
    @param  nMaskLen   mask length
    """
    res = ssw.ssw_align(qProfile, rNum, ct.c_int32(nRLen), nOpen, nExt, nFlag, 0, 0, nMaskLen)

    nScore = res.contents.nScore
    nScore2 = res.contents.nScore2
    nRefBeg = res.contents.nRefBeg
    nRefEnd = res.contents.nRefEnd
    nQryBeg = res.contents.nQryBeg
    nQryEnd = res.contents.nQryEnd
    nRefEnd2 = res.contents.nRefEnd2
    lCigar = [res.contents.sCigar[idx] for idx in range(res.contents.nCigarLen)]
    nCigarLen = res.contents.nCigarLen
    ssw.align_destroy(res)

    return (nScore, nScore2, nRefBeg, nRefEnd, nQryBeg, nQryEnd, nRefEnd2, nCigarLen, lCigar)


def buildPath(q, r, nQryBeg, nRefBeg, lCigar):
    """
    build cigar string and align path based on cigar array returned by ssw_align
    @param  q   query sequence
    @param  r   reference sequence
    @param  nQryBeg   begin position of query sequence
    @param  nRefBeg   begin position of reference sequence
    @param  lCigar   cigar array
    """
    sCigarInfo = 'MIDNSHP=X'
    sCigar = ''
    sQ = ''
    sA = ''
    sR = ''
    nQOff = nQryBeg
    nROff = nRefBeg
    for x in lCigar:
        n = x >> 4
        m = x & 15
        if m > 8:
            c = 'M'
        else:
            c = sCigarInfo[m]
        sCigar += str(n) + c

        if c == 'M':
            sQ += q[nQOff : nQOff+n]
            sA += ''.join(['|' if q[nQOff+j] == r[nROff+j] else '*' for j in xrange(n)])
            sR += r[nROff : nROff+n]
            nQOff += n
            nROff += n
        elif c == 'I':
            sQ += q[nQOff : nQOff+n]
            sA += ' ' * n
            sR += '-' * n
            nQOff += n
        elif c == 'D':
            sQ += '-' * n
            sA += ' ' * n
            sR += r[nROff : nROff+n]
            nROff += n

    return sCigar, sQ, sA, sR




def main(args):
    lEle = []
    dRc = {} 
    dEle2Int = {}
    dInt2Ele = {}
    if False == args.bProtien:
# init DNA score matrix
        if not args.sMatrix:
            lEle = ['A', 'C', 'G', 'T', 'N']
            dRc = {'A':'C', 'C':'G', 'G':'C', 'T':'A', 'a':'C', 'c':'G', 'g':'C', 't':'A'} 
            for i,ele in enumerate(lEle):
                dEle2Int[ele] = i
                dEle2Int[ele.lower()] = i
                dInt2Ele[i] = ele
            nEleNum = len(lEle)
            lScore = [0 for i in xrange(nEleNum**2)]
            for i in xrange(nEleNum-1):
                for j in xrange(nEleNum-1):
                    if lEle[i] == lEle[j]:
                        lScore[i*nEleNum+j] = args.nMatch
                    else:
                        lScore[i*nEleNum+j] = -args.nMismatch
        else:
            lEle, dEle2Int, dInt2Ele, lScore = ssw.read_matrix(args.sMatrix)
    else:
# load AA score matrix
        if not args.sMatrix:
            lEle = 'A   R   N   D   C   Q   E   G   H   I   L   K   M   F   P   S   T   W   Y   V   B   Z   X   *'.split()
            for i,ele in enumerate(lEle):
                dEle2Int[ele] = i
                dEle2Int[ele.lower()] = i
                dInt2Ele[i] = ele
            nEleNum = len(lEle)
            lScore = ssw_lib.lBlosum50
        else:
            # assume the format of the input score matrix is the same as that of http://www.ncbi.nlm.nih.gov/Class/FieldGuide/BLOSUM62.txt
            lEle, dEle2Int, dInt2Ele, lScore = ssw.read_matrix(args.sMatrix)

    if args.bBest and args.bProtien:
        print >> sys.stderr, 'Reverse complement alignment is not available for protein sequences.'

# translate score matrix to ctypes
    mat = (len(lScore) * ct.c_int8) ()
    mat[:] = lScore
# set flag
    nFlag = 0
    if args.bPath:
        nFlag = 2
# print sam head
    if args.bSam and args.bHeader and args.bPath:
        print '@HD\tVN:1.4\tSO:queryname'
        for sRId,sRSeq,_ in read(sTarget):
            print '@SQ\tSN:{}\tLN:{}'.format(sRId, len(sRSeq))
    elif args.bSam and not args.bPath:
        print >> sys.stderr, 'SAM format output is only available together with option -c.\n'
        args.bSam = False

    ssw = ssw_lib.CSsw(args.sLibPath)
# iterate query sequence
    for sQId,sQSeq,sQQual in read(args.query):
# build query profile
        qNum = to_int(sQSeq, lEle, dEle2Int)
        qProfile = ssw.ssw_init(qNum, ct.c_int32(len(sQSeq)), mat, len(lEle), 2)
# build rc query profile
        if args.bBest and not args.bProtien:
            sQRcSeq = ''.join([dRc[x] for x in sQSeq[::-1]])
            qRcNum = to_int(sQRcSeq, lEle, dEle2Int)
            qRcProfile = ssw.ssw_init(qRcNum, ct.c_int32(len(sQSeq)), mat, len(lEle), 2)
# set mask len
        if len(sQSeq) > 30:
            nMaskLen = len(sQSeq) / 2
        else:
            nMaskLen = 15

# iter target sequence
        for sRId,sRSeq,_ in read(args.target):
            rNum = to_int(sRSeq, lEle, dEle2Int)

# format ofres: (nScore, nScore2, nRefBeg, nRefEnd, nQryBeg, nQryEnd, nRefEnd2, nCigarLen, lCigar)
            res = align_one(ssw, qProfile, rNum, len(sRSeq), args.nOpen, args.nExt, nFlag, nMaskLen)
# align rc query
            resRc = None
            if args.bBest and not args.bProtien:
                resRc = align_one(ssw, qRcProfile, rNum, len(sRSeq), args.nOpen, args.nExt, nFlag, nMaskLen)

# build cigar and trace back path
            strand = 0
            if resRc == None or res[0] > resRc[0]:
                resPrint = res
                strand = 0
                sCigar, sQ, sA, sR = buildPath(sQSeq, sRSeq, res[4], res[2], res[8])
            else:
                resPrint = resRc
                strand = 1
                sCigar, sQ, sA, sR = buildPath(sQRcSeq, sRSeq, resRc[4], resRc[2], resRc[8])

            print '\t'.join([sQId, sRId, sCigar, sQ, sA, sR])
            print sQ
            print sA
            print sR
            print res

# print results
            if not args.bSam:
                print 'target_name: {}\nquery_name: {}\noptimal_alignment_score: {}\t'.format(sRId, sQId, resPrint[0]),
                if resPrint[1] > 0:
                    print 'suboptimal_alignment_score: {}\t'.format(resPrint[1]),
                if strand == 0:
                    print 'strand: +\t',
                else: 
                    print 'strand: -\t',
                if resPrint[2] + 1:
                    print 'target_begin: {}\t'.format(resPrint[2] + 1),
                print 'target_end: {}\t'.format(resPrint[3] + 1),
                if resPrint[4] + 1:
                    print 'query_begin: {}\t'.format(resPrint[4] + 1),
                print 'query_end: {}\n'.format(resPrint[5] + 1)
                if resPrint[-2] > 0:
                    n1 = 1 + resPrint[2]
                    n2 = min(60,len(sR)) + resPrint[2] - sR.count('-',0,60)
                    n3 = 1 + resPrint[4]
                    n4 = min(60,len(sQ)) + resPrint[4] - sQ.count('-',0,60)
                    for i in range(0, len(sQ), 60):
                        print 'Target:{:>8}\t{}\t{}'.format(n1, sR[i:i+60], n2)
                        n1 = n2 + 1
                        n2 = n2 + min(60,len(sR)-i-60) - sR.count('-',i+60,i+120)

                        print '{: ^15}\t{}'.format('', sA[i:i+60])

                        print 'Query:{:>9}\t{}\t{}\n'.format(n3, sQ[i:i+60], n4)
                        n3 = n4 + 1
                        n4 = n4 + min(60,len(sQ)-i-60) - sQ.count('-',i+60,i+120)
            else:
                print "{}\t".format(sQId),
                if resPrint[0] == 0:
                    print "4\t*\t0\t255\t*\t*\t0\t0\t*\t*",
                else:
                    mapq = int(-4.343 * math.log(1-abs(resPrint[0]-resPrint[1])/float(resPrint[0])))
                    mapq = int(mapq + 4.99);
                    if mapq >= 254:
                        mapq = 254
                    if strand == 1:
                        print '16\t',
                    else:
                        print '0\t',
                    print '{}\t{}\t{}\t'.format(sRId, resPrint[2]+1, mapq),
                    print sCigar,
                    print '\t*\t0\t0\t',
                    print sQSeq[resPrint[4]:resPrint[5]+1] if strand==0 else sQRcSeq[resPrint[4]:resPrint[5]+1],
                    print '\t',
                    if sQQual:
                        if strand == 0:
                            print sQQual[resPrint[4]:resPrint[5]+1],
                        else:
                            print sQQual[-resPrint[4]-1:-resPrint[5]-1:-1]
                    else:
                        print '*',

                    print '\tAS:i:{}'.format(resPrint[0]),
                    print '\tNM:i:{}\t'.format(len(sA)-sA.count('|')),
                    if resPrint[1] > 0:
                        print 'ZS:i:{}'.format(resPrint[1])
                    else:
                        print


        ssw.init_destroy(qProfile)
        if args.bBest and not args.bProtien:
            ssw.init_destroy(qRcProfile)


# modifyed main function by Yuichi Shiraishi
def main2(query, target, min_score, nMatch = 2, nMismatch = 2, nOpen = 3, nExt = 1):

    lEle = []
    dRc = {} 
    dEle2Int = {}
    dInt2Ele = {}

    # init DNA score matrix
    lEle = ['A', 'C', 'G', 'T', 'N']
    for i,ele in enumerate(lEle):
        dEle2Int[ele] = i
        dEle2Int[ele.lower()] = i
        dInt2Ele[i] = ele
    nEleNum = len(lEle)
    lScore = [0 for i in xrange(nEleNum**2)]
    for i in xrange(nEleNum-1):
        for j in xrange(nEleNum-1):
            if lEle[i] == lEle[j]:
                lScore[i*nEleNum+j] = nMatch
            else:
                lScore[i*nEleNum+j] = - nMismatch

    # translate score matrix to ctypes
    mat = (len(lScore) * ct.c_int8) ()
    mat[:] = lScore

    # set flag
    nFlag = 0

    sLibPath = ""
    for ld_path in os.environ["LD_LIBRARY_PATH"].split(':'):
        # print ld_path
        if os.path.exists(ld_path + "/libssw.so"):
            sLibPath = ld_path # + "/libssw.so"
            break
    if sLibPath == "":
        print >> sys.stderr, "cannot find libssw.so in LD_LIBRARY_PATH"
        sys.exit(1)

    ssw = ssw_lib.CSsw(sLibPath)

    type2count = {"splice_junction_positive": 0, "splice_junction_negative": 0,
                  "intron_retention_positive": 0, "intron_retention_negative": 0}

    # iterate query sequence
    for sQId,sQSeq,sQQual in read(query):

        # build query profile
        qNum = to_int(sQSeq, lEle, dEle2Int)
        qProfile = ssw.ssw_init(qNum, ct.c_int32(len(sQSeq)), mat, len(lEle), 2)

        # set mask len
        if len(sQSeq) > 30:
            nMaskLen = len(sQSeq) / 2
        else:
            nMaskLen = 15


        # iter target sequence

        type2max_score = {"splice_junction_positive": 0, "splice_junction_negative": 0,
                          "intron_retention_positive": 0, "intron_retention_negative": 0}

        for sRId,sRSeq,_ in read(target):
            rNum = to_int(sRSeq, lEle, dEle2Int)

            # format ofres: (nScore, nScore2, nRefBeg, nRefEnd, nQryBeg, nQryEnd, nRefEnd2, nCigarLen, lCigar)
            res = align_one(ssw, qProfile, rNum, len(sRSeq), nOpen, nExt, nFlag, nMaskLen)

            # build cigar and trace back path
            strand = 0
            resPrint = res
            sCigar, sQ, sA, sR = buildPath(sQSeq, sRSeq, res[4], res[2], res[8])

            """
            print '\t'.join([sQId, sRId, sCigar, sQ, sA, sR])
            print sQ
            print sA
            print sR
            print res

            """

            temp_type = re.sub(r'_\d+$', '', sRId)
            if int(resPrint[0]) > type2max_score[temp_type]:
                type2max_score[temp_type] = int(resPrint[0])
     
            """    
            if int(resPrint[0]) > temp_max_score:
                temp_max_score = int(resPrint[0])
                temp_max_target = sRId
            """

        tnum = 0
        target_max_score = 0
        target_second_max_score = 0
        target_max_type = 0
        for ttype, tmax_score in sorted(type2max_score.items(), key = lambda x: x[1], reverse=True):
            if tnum == 0:
                target_max_score = tmax_score
                target_max_type = ttype
            elif tnum == 1:
                target_second_max_score = tmax_score
            else: continue
            tnum = tnum + 1
         
        if target_max_score <= min_score: continue
        if target_max_score == target_second_max_score: continue
 
        type2count[target_max_type] = type2count[target_max_type] + 1
 
    return type2count

"""
if __name__ == '__main__':

    parser = ap.ArgumentParser()
    parser.add_argument('-l', '--sLibPath', default='', help='path of libssw.so')
    parser.add_argument('-m', '--nMatch', type=int, default=2, help='a positive integer as the score for a match in genome sequence alignment. [default: 2]')
    parser.add_argument('-x', '--nMismatch', type=int, default=2, help='a positive integer as the score for a mismatch in genome sequence alignment. [default: 2]')
    parser.add_argument('-o', '--nOpen', type=int, default=3, help='a positive integer as the penalty for the gap opening in genome sequence alignment. [default: 3]')
    parser.add_argument('-e', '--nExt', type=int, default=1, help='a positive integer as the penalty for the gap extension in genome sequence alignment. [default: 1]')
    parser.add_argument('-p', '--bProtien', action='store_true', help='Do protein sequence alignment. Without this option, the ssw_test will do genome sequence alignment. [default: False]')
    parser.add_argument('-a', '--sMatrix', default='', help='a file for either Blosum or Pam weight matrix. [default: Blosum50]')
    parser.add_argument('-c', '--bPath', action='store_true', help='Return the alignment path. [default: False]')
    parser.add_argument('-f', '--nThr', default=0, help='a positive integer. Only output the alignments with the Smith-Waterman score >= N.')
    parser.add_argument('-r', '--bBest', action='store_true', help='The best alignment will be picked between the original read alignment and the reverse complement read alignment. [default: False]')
    parser.add_argument('-s', '--bSam', action='store_true', help='Output in SAM format. [default: no header]')
    parser.add_argument('-header', '--bHeader', action='store_true', help='If -s is used, include header in SAM output.')
    parser.add_argument('target', help='targe file')
    parser.add_argument('query', help='query file')
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit()
    args = parser.parse_args()

    t1 = ti.default_timer()
    main(args)
    t2 = ti.default_timer()
    print >> sys.stderr, 'CPU time: {} seconds'.format(t2 - t1)

"""

if __name__ == '__main__':
    main2(sys.argv[1], sys.argv[2], int(sys.argv[3]))



#! /usr/bin/env python

import sys, os, subprocess
import intron_db
import annot_utils.boundary

def simple_count_main(args):

    import simple_count

    output_dir = os.path.dirname(args.output_file)
    if output_dir != "" and not os.path.exists(output_dir):
       os.makedirs(output_dir)


    # intron_db.generate_edge_bed(args.ref_gene_file, output_prefix + ".refGene.edge.bed", args.chr_name_list)
    # intron_db.generate_intron_retention_list(args.ref_gene_file, args.output_file + ".refGene.edge.bed", 
    #                                          "1,0", "0,1", args.chr_name_list)

    annot_utils.boundary.make_boundary_info(args.output_file + ".refGene.edge.bed.gz", args.genome_id, args.grc, "1,0", "0,1")

    intron_db.broaden_edge(args.output_file + ".refGene.edge.bed.gz", 
                           args.output_file + ".refGene.edge_broaden.bed",
                           args.intron_retention_check_size)

    simple_count.filterImproper(args.bam_file, args.output_file + ".filt.bam", args.mapping_qual_thres, args.keep_improper_pair)


    hout = open(args.output_file + ".filt.bed12", 'w')
    s_ret = subprocess.check_call(["bedtools", "bamtobed", "-bed12", "-i", args.output_file + ".filt.bam"], stdout = hout)
    hout.close()

    if s_ret != 0:
        print >> sys.stderr, "error in generating filt.bed12"
        sys.exit(1)


    hout = open(args.output_file + ".edge.bed", 'w')
    s_ret = subprocess.check_call(["bedtools", "intersect", "-a", args.output_file + ".filt.bed12", "-b", 
                             args.output_file + ".refGene.edge.bed.gz", "-split", "-wo"], stdout = hout) 
    hout.close()

    if s_ret != 0:
        print >> sys.stderr, "error in generating edge.bed"
        sys.exit(1)


    hout = open(args.output_file + ".edge_broaden.bed", 'w')
    s_ret = subprocess.check_call(["bedtools", "intersect", "-a", args.output_file + ".filt.bed12", "-b", 
                             args.output_file + ".refGene.edge_broaden.bed", "-split", "-wo"], stdout = hout) 
    hout.close()

    if s_ret != 0:
        print >> sys.stderr, "error in generating edge_broaden.bed"
        sys.exit(1)

    simple_count.summarize_edge(args.output_file + ".edge.bed",
                                args.output_file + ".edge_broaden.bed",
                                args.output_file + ".unsorted",
                                args.intron_retention_check_size)

    # print header
    hout = open(args.output_file, 'w')
    print >> hout, '\t'.join(["Chr", "Boundary_Pos", "Gene_Symbol", "Motif_Type", "Strand", 
                              "Junction_List", "Gene_ID_List", "Exon_Num_List", "Edge_Read_Count", "Intron_Retention_Read_Count"])
    hout.close()

    # sort the result
    hout = open(args.output_file, 'a')
    subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k3,3n", args.output_file + ".unsorted"], stdout = hout)
    hout.close()

    if not args.debug:
        subprocess.check_call(["rm", "-rf", args.output_file + ".filt.bam"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".filt.bed12"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".refGene.edge.bed.gz"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".refGene.edge.bed.gz.tbi"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".refGene.edge_broaden.bed"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".edge.bed"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".edge_broaden.bed"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".unsorted"])


def allele_count_main(args):

    import mutation, allele_count, pyssw

    output_dir = os.path.dirname(args.output_file)
    if output_dir != "" and not os.path.exists(output_dir):
       os.makedirs(output_dir)

    # intron_db.generate_intron_retention_list(args.ref_gene_file, args.output_file + ".intron_retention_list.bed", 
    #                                          args.donor_size, args.acceptor_size, args.chr_name_list)

    annot_utils.boundary.make_boundary_info(args.output_file + ".refGene.edge.bed.gz", args.genome_id, args.grc, args.donor_size, args.acceptor_size)

    mutation.anno2bed(args.mutation_file, args.output_file + ".mutation_list.bed")

    hout = open(args.output_file + ".mutation_list.overlap.bed", 'w')
    subprocess.check_call(["bedtools", "intersect", "-a", args.output_file + ".mutation_list.bed",
                     "-b", args.output_file + ".refGene.edge.bed.gz", "-wa", "-wb"], stdout = hout)
    hout.close()
    
    cnum = 0
    hout = open(args.output_file, 'w')
    # print header
    print >> hout, '\t'.join(["Gene_Symbol", "Chr_Mut", "Start_Mut", "End_Mut", "Ref_Mut", "Alt_Mut", 
                              "Chr_Motif", "Start_Motif", "End_Motif", "Type_Motif", "Strand_Motif",
                              "Splice_Junction_Negative", "Splice_Junction_Positive",
                              "Intron_Retention_Negative", "Intron_Retention_Positive"])


    with open(args.output_file + ".mutation_list.overlap.bed", 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            mut_chr, mut_start, mut_end, mut_ref, mut_alt = F[0], int(F[1]) + 1, int(F[2]), F[3], F[4]
            motif_chr, motif_start, motif_end, motif_type, motif_strand, junc_list = F[5], int(F[6]) + 1, int(F[7]), F[9], F[10], F[11]
            motif_gene = F[8]

            # generate template sequence (splicing junction with and/or without mutation, intron retention with and/or withoug mutation)
            allele_count.generate_template_seq(args.output_file + ".tmp.template_seq.fa" + str(cnum),
                                               args.reference, mut_chr, mut_start, mut_end, mut_ref, mut_alt,
                                               motif_chr, motif_start, motif_end, motif_type, motif_strand,
                                               junc_list, args.donor_size, args.acceptor_size, args.template_size)

            # extract short reads from bam files around the exon-intron boundary
            allele_count.extract_read_around_boundary(args.bam_file, args.output_file + ".tmp.read_seq.fa" + str(cnum),
                                                      args.reference, motif_chr, motif_start, motif_end, args.read_search_margin)

            # perform smith-waterman alignment check
            type2count = pyssw.main2(args.output_file + ".tmp.read_seq.fa" + str(cnum), args.output_file + ".tmp.template_seq.fa" + str(cnum), 
                                     4 * args.template_size - args.template_score_margin)

            print >> hout, '\t'.join([motif_gene, mut_chr, str(mut_start), str(mut_end), mut_ref, mut_alt, 
                             motif_chr, str(motif_start), str(motif_end), motif_type, motif_strand,
                             str(type2count["splice_junction_negative"]), str(type2count["splice_junction_positive"]),
                             str(type2count["intron_retention_negative"]), str(type2count["intron_retention_positive"])])
            
            if not args.debug:
                subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.template_seq.fa" + str(cnum)])
                subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.read_seq.fa" + str(cnum)])

            cnum = cnum + 1

    hout.close()

    if not args.debug:
        subprocess.check_call(["rm", "-rf", args.output_file + ".refGene.edge.bed.gz"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".refGene.edge.bed.gz.tbi"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".mutation_list.bed"])
        subprocess.check_call(["rm", "-rf", args.output_file + ".mutation_list.overlap.bed"]) 


def merge_control_main(args):

    # make directory for output if necessary
    if os.path.dirname(args.output_file) != "" and not os.path.exists(os.path.dirname(args.output_file)):
        os.makedirs(os.path.dirname(args.output_file))

    hin = open(args.intron_retention_list, 'r')
    hout = open(args.output_file + ".unsorted", 'w')

    header2ind = {}
    with open(args.intron_retention_list, 'r') as hin:
        for line in hin:

            intron_retention_file = line.rstrip('\n')
            with open(intron_retention_file, 'r') as hin2:
           
                header = hin2.readline().rstrip('\n').split('\t')
                for i, cname in enumerate(header):
                    header2ind[cname] = i

                for line2 in hin2:

                    F = line2.rstrip('\n').split('\t')
                    intron_ratio = 0
                    read_count = F[header2ind["Edge_Read_Count"]] + ',' + F[header2ind["Intron_Retention_Read_Count"]]
                    if F[header2ind["Edge_Read_Count"]] != "0":
                        intron_ratio = float(F[header2ind["Intron_Retention_Read_Count"]]) / float(F[header2ind["Edge_Read_Count"]])
       
                    key = '\t'.join(F[:8]) 
                    if intron_ratio >= args.ratio_thres: 
                        print >> hout, key + '\t' + str(round(intron_ratio, 3)) + '\t' + read_count
                

    hout = open(args.output_file + ".sorted", 'w')
    s_ret = subprocess.check_call(["sort", "-k1,1", "-k2,2n", args.output_file + ".unsorted"], stdout = hout)
    hout.close()

    if s_ret != 0:
        print >> sys.stderr, "Error in sorting merged junction file"
        sys.exit(1)

    hout = open(args.output_file + ".merged", 'w')
    with open(args.output_file + ".sorted", 'r') as hin:
        temp_key = ""
        temp_ratio = []
        for line in hin:
            F = line.rstrip('\n').split('\t')
            key = '\t'.join(F[:8])
            ratio = F[8]
            read_count = F[9]
            if key != temp_key:
                if temp_key != "":
                    if len(temp_ratio) >= args.sample_num_thres:
                        print >> hout, temp_key + '\t' + ';'.join(temp_ratio) + '\t' + ';'.join(temp_read_count) 
                temp_key = key
                temp_ratio = []
                temp_read_count = []
            else:
                temp_ratio.append(str(ratio))
                temp_read_count.append(read_count)

        if key != temp_key:
            if temp_key != "":
                if len(temp_ratio) >= sample_num_thres:
                    print >> hout, temp_key + '\t' + ';'.join(temp_ratio) + '\t' + ';'.join(temp_read_count)


    hout = open(args.output_file, 'w')
    s_ret = subprocess.check_call(["bgzip", "-f", "-c", args.output_file + ".merged"], stdout = hout)
    hout.close()

    if s_ret != 0:
        print >> sys.stderr, "Error in compression merged junction file"
        sys.exit(1)


    s_ret = subprocess.check_call(["tabix", "-p", "vcf", args.output_file])
    if s_ret != 0:
        print >> sys.stderr, "Error in indexing merged junction file"
        sys.exit(1)

    subprocess.check_call(["rm", "-f", args.output_file + ".unsorted"])
    subprocess.check_call(["rm", "-f", args.output_file + ".sorted"])
    subprocess.check_call(["rm", "-f", args.output_file + ".merged"])


def filter_main(args):

    import filter
    filter.filter_intron_retention(args.intron_retention_file, args.output_file, args.pooled_control_file, args.num_thres, args.ratio_thres)


def associate_main(args):
    
    import mutation, associate

    is_sv = True if args.sv else False

    if is_sv == False:

        associate.generate_mutation_target(args.intron_retention_file, args.output_file + ".target_list.bed",
                                           args.output_file + ".intron_retention_file.header", args.donor_size, args.acceptor_size)

        # convert annovar format to vcf
        if args.mutation_format == "anno":
            mutation.anno2vcf(args.mutation_file, args.output_file + ".tmp.mutation.unsorted.vcf", args.reference)
        else:
            mutation.remove_vcf_header(args.mutation_file, args.output_file + ".tmp.mutation.unsorted.vcf")

        hout = open(args.output_file + ".tmp.mutation.sorted.vcf", 'w')
        s_ret = subprocess.check_call(["sort", "-k1,1", "-k2,2n", args.output_file + ".tmp.mutation.unsorted.vcf"], stdout = hout)
        hout.close()

        if s_ret != 0:
            print >> sys.stderr, "Error in sorting vcf file"
            sys.exit(1)

        mutation.vcf2bed(args.output_file + ".tmp.mutation.sorted.vcf", args.output_file + ".tmp.mutation.sorted.vcf.bed")

        # mutation.anno2bed(args.mutation_file, args.output_file + ".mutation_list.bed")
       
        hout = open(args.output_file + ".mutation_list.associate.bed", 'w')
        subprocess.check_call(["bedtools", "intersect", "-a", args.output_file + ".tmp.mutation.sorted.vcf.bed",
                         "-b", args.output_file + ".target_list.bed", "-wa", "-wb"], stdout = hout)
        hout.close()

        associate.process_result(args.output_file + ".mutation_list.associate.bed", 
                                 args.output_file + ".intron_retention_file.header", 
                                 args.output_file, args.donor_size, args.acceptor_size)

        if args.debug == False:
            subprocess.check_call(["rm", "-rf", args.output_file + ".target_list.bed"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".intron_retention_file.header"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.mutation.unsorted.vcf"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.mutation.sorted.vcf"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.mutation.sorted.vcf.bed"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".mutation_list.associate.bed"])


    else:
        
        associate.generate_sv_target(args.intron_retention_file, args.output_file + ".target_list.bed",
                                     args.output_file + ".intron_retention_file.header", args.intron_margin)

        mutation.genosv2bed(args.mutation_file, args.output_file + ".tmp.unsorted.bedpe")

        hout = open(args.output_file + ".tmp.bedpe", 'w')
        s_ret = subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k3,3n", "-k4,4", "-k5,5n", "-k6,6n", args.output_file + ".tmp.unsorted.bedpe"], stdout = hout)
        hout.close()

        if s_ret != 0:
            print >> sys.stderr, "Error in sorting bedpe file"
            sys.exit(1)

        hout = open(args.output_file + ".sv_list.associate.bed", 'w')
        subprocess.check_call(["bedtools", "intersect", "-a", args.output_file + ".tmp.bedpe",
                         "-b", args.output_file + ".target_list.bed", "-wa", "-wb"], stdout = hout)
        hout.close()

        associate.process_result_sv(args.output_file + ".sv_list.associate.bed",
                                    args.output_file + ".intron_retention_file.header",
                                    args.output_file)

        if args.debug == False:
            subprocess.check_call(["rm", "-rf", args.output_file + ".target_list.bed"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".intron_retention_file.header"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.unsorted.bedpe"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".tmp.bedpe"])
            subprocess.check_call(["rm", "-rf", args.output_file + ".sv_list.associate.bed"])





#! /usr/bin/env python

from run import *
import argparse

def create_parser():

    parser = argparse.ArgumentParser(prog = "intron_retention_utils")

    parser.add_argument("--version", action = "version", version = "intron_retention_utils-0.4.0rc1")

    subparsers = parser.add_subparsers()

    ##########
    # simple intron retention count 

    simple_count = subparsers.add_parser("simple_count",
                                         help = "simple intron retention count program")

    simple_count.add_argument("bam_file", metavar = "sequence.bam", default = None, type = str,
                              help = "the path to the bam file")

    simple_count.add_argument("output_file", metavar = "output_file", default = None, type = str, 
                              help = "the path to the output")

    simple_count.add_argument("--grc", default = False, action = 'store_true',
                              help = "convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    simple_count.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                              help = "the genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    simple_count.add_argument("--intron_retention_check_size", metavar = "intron_retention_check_size", default = 10, type = int,
                              help = "exon and intron region size to be covered by putative intron reads (default: %(default)s)")

    simple_count.add_argument("--mapping_qual_thres", metavar = "mapping_qual_thres", default= 20, type=int,
                              help = "threshold for mapping quality for calculating base counts (default: %(default)s)")

    simple_count.add_argument("--keep_improper_pair", action = 'store_true', default = False,
                              help = "keep improper paired reads (activate for single end reads) (default: %(default)s)")

    simple_count.add_argument("--debug", default = False, action = 'store_true', help = "keep intermediate files")

    simple_count.set_defaults(func = simple_count_main)

    ##########
    # allele specific intron retention count

    allele_count = subparsers.add_parser("allele_count",
                                         help = "intron retention count incorporating somatic mutations")

    allele_count.add_argument("bam_file", metavar = "sequence.bam", default = None, type = str,
                              help = "the path to the bam file")

    allele_count.add_argument("mutation_file", metavar = "mutation.txt", default = None, type = str,
                              help = "the path to the mutation list file")

    allele_count.add_argument("output_file", metavar = "output.txt", default = None, type = str,
                              help = "the path to output file")

    allele_count.add_argument("--reference", metavar = "reference.fa", default = None, type = str, required = True,
                              help = "the path to the reference genome file")

    allele_count.add_argument("--grc", default = False, action = 'store_true',
                              help = "convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    allele_count.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                              help = "the genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    allele_count.add_argument("--donor_size", metavar = "donor_size", default = "2,6", type = str,
                              help = "splicing donor site size (exonic region size, intronic region size) (default: %(default)s)")

    allele_count.add_argument("--acceptor_size", metavar = "acceptor_size", default = "8,1", type = str,
                              help = "splicing donor site size (intronic region size, exonic region size) (default: %(default)s)")

    allele_count.add_argument("--template_size", metavar = "check_size", default = 10, type = int,
                              help = "the template sequence sizes for checking intron retention (default: %(default)s)")

    allele_count.add_argument("--template_score_margin", metavar = "check_size", default = 3, type = int,
                              help = "the margin size for checking template match score (default: %(default)s)")

    allele_count.add_argument("--read_search_margin", metavar = "read_search_margin", default = 10, type = int,
                              help = "margin size for extracting short reads around exon-intron junctions (default: %(default)s)")

    allele_count.add_argument("--debug", default = False, action = 'store_true', help = "keep intermediate files")

    allele_count.set_defaults(func = allele_count_main)

    ##########
    # merge control 
    merge_control = subparsers.add_parser("merge_control",
                                          help = "merge, compress and index control intron retention files generated by simple_count command")

    merge_control.add_argument("intron_retention_list", metavar = "intron_retention_list.txt", default = None, type = str,
                               help = "intron retention file path list")

    merge_control.add_argument("output_file", metavar = "merge_control.bedpe.gz", default = None, type = str,
                               help = "the path of the bgzip-compressed and tabix-indexed output file")


    merge_control.add_argument("--ratio_thres", type = float, default = 0.05,
                               help = "register intron retentions whose ratios (Intron_Retention_Read_Count / Edge_Read_Count) \
                               are above this value in at least the specified number of files by sample_num_thres (default: %(default)s)")

    merge_control.add_argument("--sample_num_thres", type = int, default = 2,
                               help = "register intron retentions whose ratios (Intron_Retention_Read_Count / Edge_Read_Count) \
                               are above the specifed value by ratio_thres in at least the specified number of sample by this parameter \
                                (default: %(default)s)")

    merge_control.set_defaults(func = merge_control_main)

    ##########
    # filter

    filter = subparsers.add_parser("filter",
                                   help = "filter out intron retentions that do not satisty specified conditions")

    filter.add_argument("intron_retention_file", metavar = "intron_retention.txt", default = None, type = str,
                        help = "the path to intron retention file generated by simple_count command")

    filter.add_argument("output_file", metavar = "output.txt", default = None, type = str,
                        help = "the path to the output file")

    filter.add_argument("--num_thres", type = int, default = 3,
                        help = "remove intron retentions whose supporting read numbers are below this value (default: %(default)s)")

    filter.add_argument("--ratio_thres", type = int, default = 0.05,
                        help = "remove intron retentions whose whose ratios (Intron_Retention_Read_Count / Edge_Read_Count) \
                        are below this value (default: %(default)s)")
                               
    filter.add_argument("--pooled_control_file", default = None, type = str,
                        help = "the path to control data created by merge_control (default: %(default)s)")

    filter.set_defaults(func = filter_main)

    ##########
    # associate

    associate = subparsers.add_parser("associate",
                                      help = "associate intron retentions with mutations")

    associate.add_argument("intron_retention_file", metavar = "intron_retention.txt", default = None, type = str,
                           help = "the path to intron retention file generated by simple_count command")

    associate.add_argument("mutation_file", metavar = "mutation.txt", default = None, type = str,
                           help = "the path to the mutation list file (vcf or annovar format)")

    associate.add_argument("output_file", metavar = "output_file", default = None, type = str, 
                           help = "the path to the output file")

    associate.add_argument("--donor_size", metavar = "donor_size", default = "3,6", type = str,
                           help = "splicing donor site size (exonic region size, intronic region size) (default: %(default)s)")

    associate.add_argument("--acceptor_size", metavar = "acceptor_size", default = "6,1", type = str,
                           help = "splicing donor site size (intronic region size, exonic region size) (default: %(default)s)")

    associate.add_argument('--mutation_format', choices=['vcf', 'anno'], default = 'vcf',
                           help = "the format of mutation file vcf or annovar (tsv) format (default: %(default)s)")

    associate.add_argument("--reference", metavar = "reference.fa", type = str, 
                           help = "the path to the reference genomoe sequence (necessary when --mutation format is anno)")

    associate.add_argument('--sv', action='store_true',
                           help = "analysis structural variation file")

    associate.add_argument("--intron_margin", metavar = "intron_margin", default = 10, type = int,
                           help = "the margin size of intron region for checking associations with sv breakpoints (default: %(default)s)")

    associate.add_argument("--debug", default = False, action = 'store_true', help = "keep intermediate files")

    associate.set_defaults(func = associate_main)
    ##########

    return parser




#! /usr/bin/env python

import gzip, subprocess

# this is deprecated and now this package uses annot_utils
def generate_edge_bed(ref_gene_file, output_file, chr_name_list):

    ucsc2new_chr = {}
    if chr_name_list is not None:
        with open(chr_name_list, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                ucsc2new_chr[F[0]] = F[1]

    junction2annot = {}
    ##########
    # function for processing each line of refGene.txt.gz
    def proc_line(line, type):

        F  = line.rstrip('\n').split('\t')
        
        chr = ucsc2new_chr[F[2]] if F[2] in ucsc2new_chr else F[2]
        starts = F[9].split(',')
        ends = F[10].split(',')
        strand = F[3]
        exon_num = int(F[8])
        gene = F[1]
        symbol = F[12]

        chr = chr.replace("chr", "")
    
        for i in range(1, exon_num):
            
            key = chr + '\t' + starts[i] + '\t' + str(int(starts[i]) + 1) + '\t' + '-'
            annot = symbol + '(' + gene + ').' if type == "refGene" else gene
            annot = annot + (str(i) + ".start" if strand == '+' else str(exon_num - i) + ".end")
            if key in junction2annot: 
                junction2annot[key] = junction2annot[key] + ',' + annot
            else:
                junction2annot[key] = annot
        
    
        for i in range(0, exon_num - 1):
            
            key = chr + '\t' + str(int(ends[i]) - 1) + '\t' + ends[i] + '\t' + '+' 
            annot = symbol + '(' + gene + ').' if type == "refGene" else gene
            annot = symbol + '(' + gene + ').' + (str(i) + ".end" if strand == '+' else str(exon_num - i - 1) + ".start")
            
            if key in junction2annot: 
                junction2annot[key] = junction2annot[key] + ',' + annot
            else:
                junction2annot[key] = annot
    ##########

    with gzip.open(ref_gene_file, 'r') as hin:
        for line in hin:
            proc_line(line, "refGene")

    hout = open(output_file, 'w')
    for junction in sorted(junction2annot):
        junc_info = junction.split('\t')
        print >> hout, '\t'.join(junc_info[0:3]) + '\t' + junction2annot[junction] + '\t' + '0' + '\t' + junc_info[3]
    hout.close()


def broaden_edge(input_file, output_file, margin):

    hout = open(output_file, 'w')
    with gzip.open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            if (F[4] == "donor" and F[5] == '+') or (F[4] == "acceptor" and F[5] == '-'):
                print >> hout, F[0] + '\t' + str(int(F[1]) - margin + 1) + '\t' + str(int(F[2]) + margin) + '\t' + '\t'.join(F[3:])
            else:
                print >> hout, F[0] + '\t' + str(int(F[1]) - margin) + '\t' + str(int(F[2]) + margin - 1) + '\t' + '\t'.join(F[3:])

    hout.close()



def generate_intron_retention_list(ref_gene_file, output_file, donor_size, acceptor_size, chr_name_list):

    ucsc2new_chr = {}
    if chr_name_list is not None:
        with open(chr_name_list, 'r') as hin:
            for line in hin:
                F = line.rstrip('\n').split('\t')
                ucsc2new_chr[F[0]] = F[1]

    donor_size_exon, donor_size_intron = [int(x) for x in donor_size.split(',')]
    acceptor_size_intron, acceptor_size_exon = [int(x) for x in acceptor_size.split(',')]

    key2junction, key2gene_id, key2exon_num = {}, {}, {}
    with gzip.open(ref_gene_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')

            chr = ucsc2new_chr[F[2]] if F[2] in ucsc2new_chr else F[2] 
            starts = [int(x) for x in F[9].split(',') if x != '']
            ends = [int(x) for x in F[10].split(',') if x != '']
            strand = F[3]
            exon_num = int(F[8])
            gene_id = F[1]
            symbol = F[12]

            
            for i in range(0, exon_num - 1):
                if strand == '+': # donor
                    key = '\t'.join([chr, str(ends[i] - donor_size_exon), str(ends[i] + donor_size_intron), symbol, "donor", strand])
                else: # acceptor
                    key = '\t'.join([chr, str(ends[i] - acceptor_size_exon), str(ends[i] + acceptor_size_intron), symbol, "acceptor", strand])
                
                junction = chr + ':' + str(ends[i]) + '-' + str(starts[i + 1] + 1)

                if key not in key2junction: key2junction[key] = []
                if key not in key2gene_id: key2gene_id[key] = []
                if key not in key2exon_num: key2exon_num[key] = []

                key2junction[key].append(junction)
                key2gene_id[key].append(gene_id)
                key2exon_num[key].append(str(i))


            for i in range(1, exon_num):
                if strand == '+': # acceptor 
                    key = '\t'.join([chr, str(starts[i] - acceptor_size_intron), str(starts[i] + acceptor_size_exon), symbol, "acceptor", strand])
                else: # donor 
                    key = '\t'.join([chr, str(starts[i] - donor_size_intron), str(starts[i] + donor_size_exon), symbol, "donor", strand])

                junction = chr + ':' + str(ends[i - 1]) + '-' + str(int(starts[i]) + 1)

                if key not in key2junction: key2junction[key] = []
                if key not in key2gene_id: key2gene_id[key] = []
                if key not in key2exon_num: key2exon_num[key] = []
                
                key2junction[key].append(junction) 
                key2gene_id[key].append(gene_id) 
                key2exon_num[key].append(str(i)) 


    hout = open(output_file + ".tmp", 'w')
    for key in sorted(key2junction):
        print >> hout, '\t'.join([key, ','.join(key2junction[key]), ','.join(key2gene_id[key]), ','.join(key2exon_num[key])])

    hout.close()

    hout = open(output_file, 'w')
    subprocess.check_call(["sort", "-k1,1", "-k2,2n", "-k3,3n", output_file + ".tmp"], stdout = hout)
    hout.close()

    subprocess.check_call(["rm", "-rf", output_file + ".tmp"])


